// D import file generated from 'gl3n/aabb.d'
module gl3n.aabb;
private
{
	import gl3n.linalg : Vector;
	import gl3n.math : almost_equal;
	import gl3n.util : TupleRange;
	static import std.compiler;
}
struct AABBT(type, uint dimension_ = 3)
{
	alias at = type;
	alias vec = Vector!(at, dimension_);
	alias dimension = dimension_;
	static assert(dimension > 0, "0 dimensional AABB don't exist.");
	vec min = vec(cast(at)0.0);
	vec max = vec(cast(at)0.0);
	nothrow pure @safe
	{
		this(vec min, vec max)
		{
			this.min = min;
			this.max = max;
		}
		static AABBT from_points(vec[] points)
		{
			AABBT res;
			if (points.length == 0)
			{
				return res;
			}
			res.min = points[0];
			res.max = points[0];
			foreach (v; points[1..$])
			{
				res.expand(v);
			}
			return res;
		}
		version (unittest)
		{
			private static vec sizedVec(T)(T[] values)
			{
				at[] ret;
				foreach (i; 0 .. dimension)
				{
					ret ~= cast(at)values[i];
				}
				return vec(ret);
			}
		}
		void expand(AABBT b)
		{
			foreach (i; TupleRange!(0, dimension))
			{
				if (min.vector[i] > b.min.vector[i])
					min.vector[i] = b.min.vector[i];
				if (max.vector[i] < b.max.vector[i])
					max.vector[i] = b.max.vector[i];
			}
		}
		void expand(vec v)
		{
			foreach (i; TupleRange!(0, dimension))
			{
				if (min.vector[i] > v.vector[i])
					min.vector[i] = v.vector[i];
				if (max.vector[i] < v.vector[i])
					max.vector[i] = v.vector[i];
			}
		}
		const bool intersects(AABBT box)
		{
			foreach (i; TupleRange!(0, dimension))
			{
				if (min.vector[i] >= box.max.vector[i] || max.vector[i] <= box.min.vector[i])
					return false;
			}
			return true;
		}
		const @property vec extent()
		{
			return max - min;
		}
		const @property vec half_extent()
		{
			return (max - min) / 2;
		}
		static if (dimension <= 3)
		{
			const @property real area()
			{
				vec e = extent;
				static if (dimension == 1)
				{
					return 0;
				}
				else
				{
					static if (dimension == 2)
					{
						return e.x * e.y;
					}
					else
					{
						static if (dimension == 3)
						{
							return 2.0 * (e.x * e.y + e.x * e.z + e.y * e.z);
						}
						else
						{
							static assert(dimension <= 3, "area() not supported for aabb of dimension > 3");
						}
					}
				}
			}
		}
		const @property vec center()
		{
			return (max + min) / 2;
		}
		const @property vec[] vertices()
		{
			vec[] res;
			res.length = 2 ^^ dimension;
			foreach (i; TupleRange!(0, 2 ^^ dimension))
			{
				foreach (dim; TupleRange!(0, dimension))
				{
					res[i].vector[dim] = i & 1 << dim ? max.vector[dim] : min.vector[dim];
				}
			}
			return res;
		}
		static if (std.compiler.version_major > 2 || std.compiler.version_minor >= 69)
		{
		}
		const bool opEquals(AABBT other)
		{
			return other.min == min && (other.max == max);
		}
	}
}
alias AABB3 = AABBT!(float, 3);
alias AABB2 = AABBT!(float, 2);
alias AABB = AABB3;
